/* midi - MIDI file library and Simple Software Synthesizer
 * Copyright (C) 2004 by Patrick Reynolds <reynolds@cs.duke.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef MIDIFILE_H
#define MIDIFILE_H

#include <stdio.h>
#include <glib.h>

#define TRACK(mf, t) ((Track*)g_ptr_array_index((mf)->tracks, (t)))
#define EVENT(mf, t, e) ((Event*)g_ptr_array_index(TRACK((mf), (t))->events, (e)))

typedef struct _Header Header;
struct _Header {
	int format, numtracks, division;
};

typedef struct _Track Track;
struct _Track {
	GPtrArray *events;
};

typedef struct _Event Event;
struct _Event {
	int status, pitch, velocity, start;
	char *data;
};

typedef struct _MIDIFile MIDIFile;
struct _MIDIFile {
	Header *header;
	GPtrArray *tracks;
	int tempo;
};

MIDIFile *read_midi_file(FILE *fp);
Header *read_header_chunk(FILE *fp, int length);
Track *read_track(FILE *fp, int length);
Event *read_event(FILE *fp, Event *last_event);
unsigned short read_int16(FILE *fp);
unsigned long read_int32(FILE *fp);
unsigned long read_varint(FILE *fp);

void write_midi_file(FILE *fp, MIDIFile *mf);
void write_header_chunk(FILE *fp, Header *header);
void write_track(FILE *fp, Track *track);
void write_event(FILE *fp, Event *event, Event *last_event);
void write_int16(FILE *fp, unsigned short n);
void write_int32(FILE *fp, unsigned long n);
void write_varint(FILE *fp, unsigned long n);

void merge_tracks(MIDIFile *mf);

void free_midi_file(MIDIFile *mf);
void free_header(Header *header);
void free_track(Track *track);
void free_event(Event *event);

void debug(int level, const char *fmt, ...);

#endif
